@@ -8,7 +8,7 @@ use Exporter ();
 use List::MoreUtils qw(uniq);
 use List::Util qw(max min);
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 our ( @ISA, @EXPORT_OK, %EXPORT_TAGS );
 @ISA = qw(Exporter);
@@ -23,7 +23,7 @@ my $DEG_IN_SCALE = 12;
 # SUBROUTINES
 
 sub chord_inv {
-  my ($pitch_set) = @_;
+  my ( $pitch_set, %params ) = @_;
   croak "pitch set reference required"
     unless defined $pitch_set and ref $pitch_set eq 'ARRAY';
 
@@ -42,13 +42,13 @@ sub chord_inv {
       ];
 
     # Normalize to "0th" register if lowest pitch is an octave+ out
-    # TODO this is a output/display issue, move to formatting routines if
-    # write those.
-    #   my $min_pitch = min( @{ $inversions[-1] } );
-    #   if ( $min_pitch >= $DEG_IN_SCALE ) {
-    #     my $offset = $min_pitch - $min_pitch % $DEG_IN_SCALE;
-    #     $_ -= $offset for @{ $inversions[-1] };
-    #   }
+    if ( exists $params{'pitch_norm'} and $params{'pitch_norm'} ) {
+      my $min_pitch = min( @{ $inversions[-1] } );
+      if ( $min_pitch >= $DEG_IN_SCALE ) {
+        my $offset = $min_pitch - $min_pitch % $DEG_IN_SCALE;
+        $_ -= $offset for @{ $inversions[-1] };
+      }
+    }
   }
 
   return @inversions;
@@ -99,7 +99,9 @@ sub chord_pos {
   @potentials = @ps;
   for my $i ( 1 .. $params{'octave_count'} ) {
     for my $n (@ps) {
-      push @potentials, $n + $i * $DEG_IN_SCALE;
+      my $p = $n + $i * $DEG_IN_SCALE;
+      push @potentials, $p
+        unless exists $params{'pitch_max'} and $p > $params{'pitch_max'};
     }
   }
   @potentials = uniq sort { $a <=> $b } @potentials;
@@ -122,7 +124,8 @@ sub chord_pos {
         $harmeq{ $p % $DEG_IN_SCALE }++;
       }
 
-      unless ( exists $params{'no_limit_uniq'} and $params{'no_limit_uniq'} ) {
+      unless ( exists $params{'no_limit_uniq'} and $params{'no_limit_uniq'} )
+      {
         next if keys %harmeq < $unique_pitch_count;
       }
 
@@ -175,6 +178,9 @@ sub chords2voices {
   my (@pitch_sets) = @_;
   croak "not a list of pitch sets" unless ref $pitch_sets[0] eq 'ARRAY';
 
+  # Nothing to swap, change nothing
+  return @pitch_sets if @pitch_sets < 2;
+
   my @voices;
 
   for my $vi ( 0 .. $#{ $pitch_sets[0] } ) {
@@ -239,7 +245,7 @@ stuff. If OO desired, code something up?
 
 =over 4
 
-=item B<chord_inv>( I<pitch set reference> )
+=item B<chord_inv>( I<pitch set reference>, I<list of optional paramters> ... )
 
 Generates inversions of the pitch set, returns a list of pitch sets
 (list of array references). The order of the results will be 1st
@@ -249,6 +255,17 @@ No transposition is performed, so inversions of 9ths or larger may
 result in a chord in a register above the original. If this is a
 problem, decrement the semitones in the pitch set by 12 or whatever.
 
+Parameter accepted (just one):
+
+=over 4
+
+=item B<pitch_norm> => I<0>
+
+If set and true, transposes inversions down if lowest pitch of said
+inversion is greater than the degrees in the scale.
+
+=back
+
 =item B<chord_pos>( I<pitch set reference>, I<list of optional parameters> ... )
 
 Generate different voicings of a different chord, by default in
@@ -268,38 +285,44 @@ in many, many, many different voicings for larger pitch sets.
 
 =over 4
 
-=item B<interval_adj_max> I<19>
+=item B<interval_adj_max> => I<19>
 
 Largest interval allowed between two adjacent voices, in semitones.
 
-=item B<no_limit_doublings> I<0>
+=item B<no_limit_doublings> => I<0>
 
 If set and true, allows doublings on all pitches, not just the default
 of the root pitch.
 
-=item B<no_limit_uniq> I<0>
+=item B<no_limit_uniq> => I<0>
 
 If set and true, disables the unique pitch check. That is, voicings will
 be allowed with fewer pitches than in the original pitch set.
 
-=item B<octave_count> I<2>
+=item B<octave_count> => I<2>
 
 How far above the register of the chord to generate voicings in. If set
 to a large value, the B<interval_adj_max> value may also need to be
-increased.
+increased. The B<pitch_max> parameter can be used to fine-tune the
+maximum pitch generated inside the set generated by this option.
 
-=item B<root_any> I<0>
+=item B<pitch_max> => I<inf>
+
+Maximum pitch to allow, in semitones. For fine-grained control below the
+span generated by the B<octave_count> parameter.
+
+=item B<root_any> => I<0>
 
 If set and true, allows the root pitch of the voicing to be any member
 of the original pitch set, not just the lowest of that set. Pointless if
 B<root_lock> set.
 
-=item B<root_lock> I<0>
+=item B<root_lock> => I<0>
 
 Prevent the root pitch from changing in the generated positions. Defeats
 B<root_any> option.
 
-=item B<voice_count> I<depends on pitch set passed>
+=item B<voice_count> => I<depends on pitch set passed>
 
 Use this to customize the number of voices in the different chord
 voicings. At present, only one extra voice above the number of voices
@@ -319,7 +342,7 @@ tenor from the upper voices.
 
 Accepts a pitch set list (such as returned by B<chord_pos>), transposes
 vertical chords into horizontal voices. Returns list of voices, highest
-to lowest.
+to lowest. Returns the original pitch set list if nothing to transpose.
 
 =item B<scale_deg>( )
 
